//Функция генерации обучающих данных (для динамики выходных характеристик)
function [SystemOutputChatacteristics..//Выходные характеристики системы
          ]=AdvancedModelLearningDataEvidence(SystemOutputCharacteristicsNames,...//Имена выходных характеристик системы
                                              SystemOutputCharacteristicsNamesIndexes,...//Индексы выходных характеристик системы
                                              SystemOutputCharacteristicsIndexes,...//Индексы выходных характеристик системы (в именах)
                                              SystemShemaName,...//Имя схемы системы
                                              SystemOutputCharacteristicsCompressionFunctions,...//Функция сжатия динамики
                                              SystemParametrsNames,...//Имена параметров, определяющих динамику
                                              SystemParametrsValues,...//Значения параметров, определяющих динамику
                                              absolutTolErr,..//Максимальное значение абсолютной ошибки
                                              relativeTolErr,..//Минимальное значение относительной ошибки
                                              simulationTime,..//Время симуляции
                                              timeStepMax,..//Максимальный шаг интергирования по времени
                                              Solver,..//Решатель
                                              toleranceTime..//Максимальная ошибка времени
                                              )
    //Проверяем число аргументов
    nArg = argn(2);//Число аргументов
    if nArg < 7 then
        error('Function AdvancedModelLearningDataEvidence takes 7 or more arguments (maximum 13 arguments)');
    end
    
    //Восполняем недостающие аргументы
    select nArg
    case 7 then
        absolutTolErr = [];
        relativeTolErr = [];
        simulationTime = [];
        timeStepMax = [];
        Solver = [];
        toleranceTime = [];
    case 8 then
        relativeTolErr = [];
        simulationTime = [];
        timeStepMax = [];
        Solver = [];
        toleranceTime = [];
    case 9 then
        simulationTime = [];
        timeStepMax = [];
        Solver = [];
        toleranceTime = [];
    case 10 then
        timeStepMax = [];
        Solver = [];
        toleranceTime = [];
    case 11 then
        Solver = [];
        toleranceTime = [];
    case 12 then
        toleranceTime = [];
    end
           
    //Приволим аргументы к виду матрица - строка
    SystemOutputCharacteristicsNamesIndexes = matrix(SystemOutputCharacteristicsNamesIndexes,1,-1);
    SystemOutputCharacteristicsIndexes = matrix(SystemOutputCharacteristicsIndexes, 1, -1);
    SystemOutputCharacteristicsCompressionFunctions = matrix(SystemOutputCharacteristicsCompressionFunctions, 1, -1);
        
    //Проверяем корректность аргументов
    if type(SystemOutputCharacteristicsNamesIndexes) ~= 1 then
        error('The second argument of function AdvancedModelLearningDataEvidence is matrix of integer');
    end
    if type(SystemOutputCharacteristicsIndexes) ~= 1 then
        error('The third argument of function AdvancedModelLearningDataEvidence is matrix of integer');
    end
    sizeSystemOutputCharacteristicsNamesIndexes = size(SystemOutputCharacteristicsNamesIndexes,'*');
    if sizeSystemOutputCharacteristicsNamesIndexes ~= size(SystemOutputCharacteristicsIndexes,'*') then
        error('The sizes of arguments SystemOutputCharacteristicsNamesIndexes and SystemOutputCharacteristicsIndexes of function AdvancedModelLearningDataEvidence must be equality');
    end
    if (type(SystemShemaName) ~= 10) ||..
       (size(SystemShemaName, '*') > 1) then
        error('The 4th argument of function AdvancedModelLearningDataEvidence is scalar string');
    end
    if type(SystemOutputCharacteristicsCompressionFunctions) == 17 then
        for ind=1:size(SystemOutputCharacteristicsCompressionFunctions,'*')
            if type(SystemOutputCharacteristicsCompressionFunctions{ind}) ~= 13 then
                error('The 5th argument of function AdvancedModelLearningDataEvidence is cell-array of functions');
            end
        end
    else
        error('The 5th argument of function AdvancedModelLearningDataEvidence is cell-array of functions');
    end
    if size(SystemOutputCharacteristicsCompressionFunctions,'*') ~= sizeSystemOutputCharacteristicsNamesIndexes then
        error('The number of compression functions must be equal to the number of dynamic curves (dimnesion of argument SystemOutputCharacteristicsNamesIndexes)');
    end
      
    //Загружаем схему системы
    SystemShema = xcosDiagramToScilab(SystemShemaName);
    
    //Векторизуемая функция вычислительного эксперимента
    nCurves = size(SystemParametrsValues)(1);//Число возможных динамик
    SystemOutputChatacteristics = [];
    for ind=1:nCurves
        //Проводим вычислительный эксперимент
        Dynamics = ComputingExperiment(SystemOutputCharacteristicsNames,...//Имена выходных характеристик системы
                                       SystemShema,...//Схема системы
                                       SystemParametrsNames,...//Имена параметров
                                       SystemParametrsValues(ind,:),...//Значения параметров
                                       absolutTolErr(ind),..//Максимальное значение абсолютной ошибки
                                       relativeTolErr(ind),..//Минимальное значение относительной ошибки
                                       simulationTime(ind),..//Время симуляции
                                       timeStepMax(ind),..//Максимальный шаг интергирования по времени
                                       Solver(ind),..//Решатель
                                       toleranceTime(ind)..//Максимальная ошибка времени
                                       );
        
        //Сжимаем полученные динамики
        SystemOutputChatacteristics = [SystemOutputChatacteristics;..
                                       CountDynamicsEvidences(Dynamics,...//Массив динамик
                                                              SystemOutputCharacteristicsNamesIndexes,...//Индексы имен динамик
                                                              SystemOutputCharacteristicsIndexes,...//Индексы динамик (в именах)
                                                              SystemOutputCharacteristicsCompressionFunctions...//Функция фильтра динамики
                                                              )]; 
    end
endfunction

//Расчет проекции полученной динамики
function [Evidences...//Параметры
          ]=CountDynamicsEvidences(Dynamics,...//Массив динамик
                                   DynamicsNamesIndexes,...//Индексы имен динамик
                                   DynamicsIndexes,...//Индексы динамик (в именах)
                                   DynamicCompressionFunctions...//Функция фильтра динамики
                                   )
    //Проверяем корректность аргументов
    if isempty(Dynamics) || isempty(DynamicsNamesIndexes) ||..
       isempty(DynamicsIndexes) then
        Evidences = [];
    end
    
    //Выполняем фильтрацию
    nDynamics = size(DynamicsIndexes,'*');//Число сжимаемых динамик
    Evidences = [];
    for ind=1:nDynamics//Вспомогательная функция фильтрации по индексу
        parametrs = DynamicCompressionFunctions{ind}(Dynamics(DynamicsNamesIndexes(ind)).time,...
                                                     Dynamics(DynamicsNamesIndexes(ind)).values(:,DynamicsIndexes(ind)));
        Evidences = [Evidences, parametrs];
    end
endfunction
