//Моделируем динамику
function [SystemOutputChatacteristics,..//Выходные характеристики системы
          SystemProps..//Свойства системы
          ]=ComputingExperiment(SystemOutputChatacteristicsNames,...//Имена выходных характеристик системы
                                SystemShema,...//Схема системы
                                SystemParametrsNames,...//Имена параметров
                                SystemParametrsValues,...//Значения параметров
                                absolutTolErr,..//Максимальное значение абсолютной ошибки
                                relativeTolErr,..//Минимальное значение относительной ошибки
                                simulationTime,..//Время симуляции
                                timeStepMax,..//Максимальный шаг интергирования по времени
                                Solver,..//Решатель
                                toleranceTime..//Максимальная ошибка времени
                                )//Моделирование возможных динамик
    //Проверяем число аргументов
    nArgs = argn(2);
    if nArgs < 2 then
        error('Function ComputingExperiment takes 2 or more arguments (maximum 10 arguments)');
    end
    
    //Проверяем аргументы
    select nArgs
    case 2 then
        SystemParametrsNames = [];
        SystemParametrsValues = [];
        absolutTolErr = [];
        relativeTolErr = [];
        simulationTime = [];
        timeStepMax = [];
        Solver = [];
        toleranceTime = [];
    case 3 then
        if isempty(SystemParametrsNames) then
            SystemParametrsValues = [];
            absolutTolErr = [];
            relativeTolErr = [];
            simulationTime = [];
            timeStepMax = [];
            Solver = [];
            toleranceTime = [];
        else
            if type(SystemParametrsNames) == 1 then
                absolutTolErr = SystemParametrsNames;
                SystemParametrsNames = [];
                SystemParametrsValues = [];
                relativeTolErr = [];
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('The argument absolutTolErr of function ComputingExperiment must be matrix of real');
            end
        end
    case 4 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                absolutTolErr = [];
                relativeTolErr = [];
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('System parametrs names and values must be empty together or not empty together');
            end
        else
            select type(SystemParametrsNames)
            case 1 then
                absolutTolErr = SystemParametrsNames;
                relativeTolErr = SystemParametrsValues;
                SystemParametrsNames = [];
                SystemParametrsValues = [];
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            case 10 then
                absolutTolErr = [];
                relativeTolErr = [];
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('The argument SystemParametrsNames must be matrix of string (or argument absolutTolErr of this function must be matrix of real)');
            end
        end
    case 5 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                relativeTolErr = [];
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('System parametrs names and values must be empty together or not empty together');
            end
        else
            select type(SystemParametrsNames)
            case 1 then
                simulationTime = absolutTolErr;
                absolutTolErr = SystemParametrsNames;
                relativeTolErr = SystemParametrsValues;
                SystemParametrsNames = [];
                SystemParametrsValues = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            case 10 then
                relativeTolErr = [];
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('The argument SystemParametrsNames must be matrix of string (or argument absolutTolErr of this function must be matrix of real)');
            end
        end
    case 6 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('System parametrs names and values must be empty together or not empty together');
            end
        else
            select type(SystemParametrsNames)
            case 1 then
                timeStepMax = relativeTolErr;
                simulationTime = absolutTolErr;
                absolutTolErr = SystemParametrsNames;
                relativeTolErr = SystemParametrsValues;
                SystemParametrsNames = [];
                SystemParametrsValues = [];
                Solver = [];
                toleranceTime = [];
            case 10 then
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('The argument SystemParametrsNames must be matrix of string (or argument absolutTolErr of this function must be matrix of real)');
            end
        end
    case 7 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('System parametrs names and values must be empty together or not empty together');
            end
        else
            select type(SystemParametrsNames)
            case 1 then
                Solver = simulationTime;
                timeStepMax = relativeTolErr;
                simulationTime = absolutTolErr;
                absolutTolErr = SystemParametrsNames;
                relativeTolErr = SystemParametrsValues;
                SystemParametrsNames = [];
                SystemParametrsValues = [];
                toleranceTime = [];
            case 10 then
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('The argument SystemParametrsNames must be matrix of string (or argument absolutTolErr of this function must be matrix of real)');
            end
        end
    case 8 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                Solver = [];
                toleranceTime = [];
            else
                error('System parametrs names and values must be empty together or not empty together');
            end
        else
            select type(SystemParametrsNames)
            case 1 then
                toleranceTime = timeStepMax;
                Solver = simulationTime;
                timeStepMax = relativeTolErr;
                simulationTime = absolutTolErr;
                absolutTolErr = SystemParametrsNames;
                relativeTolErr = SystemParametrsValues;
                SystemParametrsNames = [];
                SystemParametrsValues = [];
            case 10 then
                Solver = [];
                toleranceTime = [];
            else
                error('The argument SystemParametrsNames must be matrix of string (or argument absolutTolErr of this function must be matrix of real)');
            end
        end
    case 9 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                toleranceTime = [];
            else
                error('System parametrs names and values must be empty together or not empty together');
            end
        else
            if type(SystemParametrsNames) == 10 then
                toleranceTime = [];
            else
                error('The argument SystemParametrsNames must be matrix of string');
            end
        end
    end
    
    //Проверяем корректность типов данных аргументов
    if type(SystemOutputChatacteristicsNames) ~= 10 then
        error('The first argument of ComputingExperiment function is array of string');
    end
    if type(SystemShema) ~= 128 then
        error('The second argument of ComputingExperiment function is array of system sheme diagram type exemplar');
    end
    if isempty(SystemParametrsNames) && (~isempty(SystemParametrsValues)) ||..
       (~isempty(SystemParametrsNames)) && isempty(SystemParametrsValues) then
       error('System parametrs names and values must be empty together or not empty together');
    end
    if (~isempty(SystemParametrsValues)) &&..
       (type(SystemParametrsValues) ~= 1) &&..
       (type(SystemParametrsValues) ~= 10) then
       error('The argument SystemParametrsValues of ComputingExperiment function is array of real or string of scilab expressions');
       
       //Приводим заголовки и значения величин к матрице-строке
       SystemParametrsNames = matrix(SystemParametrsNames, 1, -1);
       SystemParametrsValues = matrix(SystemParametrsValues, 1, -1);
       
       //Проверяем соответсвие размеров матриц заголовков и значений величин
       if size(SystemParametrsNames) ~= size(SystemParametrsValues) then
           error('The matrix of names and values of system parametrs dimensions must be equality');
       end
    end
    TryScalarRealType(absolutTolErr,5);
    TryScalarRealType(relativeTolErr,6);
    TryScalarRealType(simulationTime,7);
    TryScalarRealType(timeStepMax,8);
    TryScalarRealType(Solver,9);
    TryScalarRealType(toleranceTime,10);
    
    //Приводим имена выходных характеристик системы к виду матрица-строка
    SystemOutputChatacteristicsNames = matrix(SystemOutputChatacteristicsNames, 1, -1);
    
    //Задаем параметры и контекст моделирования
    SystemShema.props.context = SetContext(SystemParametrsNames,...//Имена параметров
                                           SystemParametrsValues,...//Значения параметров
                                           SystemShema.props.context...//Задаваемый контекст
                                           );//Задаем параметры системы
    if ~isempty(absolutTolErr) then
        SystemShema.props.tol(1) = absolutTolErr;//Задаем абсолютную погрешность
    end
    if ~isempty(relativeTolErr) then
        SystemShema.props.tol(2) = relativeTolErr;//Задаем относительную погрешность
    end
    if ~isempty(simulationTime) then
        SystemShema.props.tf = simulationTime;//Задаем время симуляции
    end
    if ~isempty(timeStepMax) then
        SystemShema.props.tol(7) = timeStepMax;//Задаем vаксимальный шаг интегрирования по времени
    end
    if ~isempty(Solver) then
        SystemShema.props.tol(6) = Solver;//Задаем решатель
    end
    if ~isempty(toleranceTime) then
        SystemShema.props.tol(3) = toleranceTime;//Задаем погрешность для времени
    end
    SystemShema.props.tol(5) = 0.0;//Отключаем реальное время
                                      
    //Запускаем моделирование динамики
    xcos_simulate(SystemShema, 4);
    
    //Получаем значения выходных характеристик системы
    SystemOutputChatacteristics = evstr(SystemOutputChatacteristicsNames);
    
    //Возвращаем свойства системы
    SystemProps = SystemShema.props;
endfunction

//Вспомогательные функции
function [context]=SetContext(ParametrsNames,...//Имена параметров
                              ParametrsValues,...//Значения параметров
                              context...//Задаваемый контекст
                              )//задаем контекст
    //Проверяем аргументы
    if isempty(ParametrsNames) || isempty(context) then
        return;
    end
    
    //Удаляем строки, не являющиеся контекстом
    context = context(grep(context,'='));
    
    //Выделяем из контекста имена и значения величин
    context_split = strsplit(strcat(context, ';'), ['=',';']);//Выделяем имена величин и их значения
    context_split(length(context_split) == 0) = [];//Удаляем мусор
    contextNames = stripblanks(context_split(1:2:size(context_split, '*')));//Получаем имена аргументов в контексте
    contextValues = stripblanks(context_split(2:2:size(context_split, '*')));//Получаем значения аргументов в контексте
    
    //Формируем предварительно контекст
    context = contextNames + '=' + contextValues;
    
    //Отсеиваем пользовательские величины, не встречающиеся в контексте
    [_,indParametrsNames] = members(ParametrsNames,contextNames);
    ParametrsNames(indParametrsNames == 0) = [];//Удаляем имена величин, не входящие в контекст
    ParametrsValues(indParametrsNames == 0) = [];//Удаляем значения величин, не входящих в контекст
    indParametrsNames(indParametrsNames == 0) = [];//Индексы величин, входящих в контекст
    
    //Удаляем пробелы из имен величин
    ParametrsNames = stripblanks(ParametrsNames);
    
    //Приравниваем именам величин их значения
    parametrs = ParametrsNames + '=' + string(ParametrsValues);
    
    //Заполняем контекст
    context(indParametrsNames) = parametrs;
endfunction
function TryScalarRealType(val,numberVal)//Проверяем реальный скалярный тип данных
    if (size(val,'*') > 1) || (type(val) ~= 1) then
       error('The ' + numberVal + 'th argument of ComputingExperiment function are real scalar');
    end
endfunction
